import { createContext, ReactNode, useContext, useState, useEffect, useCallback } from "react";
import { apiRequest } from "@/lib/queryClient";
import { useToast } from "./use-toast";

interface User {
  id: number;
  username: string;
  email: string;
  fullName: string;
  balance: number;
  referralEarnings: number;
  gameEarnings: number;
  referralCount: number;
  level: number;
  referralCode: string;
  createdAt: string;
  avatarUrl: string | null;
  isSubscribed: boolean;
  specialFeatures?: string[];
  subscriptionTier?: number;
}

interface AuthContextType {
  user: User | null;
  isLoading: boolean;
  error: Error | null;
  login: (username: string, password: string) => Promise<void>;
  logout: () => Promise<void>;
  register: (userData: RegisterData) => Promise<void>;
  retryAuth: () => Promise<void>;
  lastAttemptTime?: number;
}

interface RegisterData {
  username: string;
  password: string;
  email: string;
  fullName: string;
  referralCode?: string;
}

const AuthContext = createContext<AuthContextType | null>(null);

export function AuthProvider({ children }: { children: ReactNode }) {
  const [user, setUser] = useState<User | null>(null);
  const [isLoading, setIsLoading] = useState(true);
  const [error, setError] = useState<Error | null>(null);
  const [lastAttemptTime, setLastAttemptTime] = useState<number>(Date.now());
  const [retryCount, setRetryCount] = useState(0);
  const { toast } = useToast();
  
  // Detect iOS 18.3.1 for fallback handling
  const [isIOS18_3_1, setIsIOS18_3_1] = useState(false);
  
  // Use dynamic import for iOS detection
  useEffect(() => {
    import('@/lib/ios18Detector').then(module => {
      // Check if we have either the legacy or new detection function
      if (module.detect18iOS) {
        const result = module.detect18iOS();
        setIsIOS18_3_1(result.isIOS18_3_1);
      } else if (module.isIOS1831) {
        // Fall back to the standard detection functions
        const isIOS1831 = module.isIOS1831();
        setIsIOS18_3_1(isIOS1831);
      } else {
        // Default to false if no detection method is available
        setIsIOS18_3_1(false);
      }
    }).catch(err => {
      console.error("Failed to import iOS 18 detector:", err);
    });
  }, []);
  
  // Helper function to activate fallback connection mode
  const activateFallbackMode = async () => {
    if (isIOS18_3_1) {
      try {
        const fallbackModule = await import('@/lib/ios18FallbackClient');
        if (fallbackModule.forceFallbackConnectionMode) {
          fallbackModule.forceFallbackConnectionMode();
        }
      } catch (err) {
        console.error("Error importing fallback client:", err);
      }
    }
  };
  
  // Enhanced auth check with fallback mechanisms for iOS devices
  const checkAuth = useCallback(async (skipCache = false) => {
    if (isLoading && retryCount > 0) {
      // Prevent multiple simultaneous auth checks
      return;
    }
    
    try {
      setIsLoading(true);
      setLastAttemptTime(Date.now());
      
      // Log auth check attempt with useful device info
      console.log(`Auth check attempt (${retryCount + 1}): ${new Date().toISOString()}`);
      
      // For iOS 18.3.1, force fallback mode first as a precaution
      if (isIOS18_3_1) {
        await activateFallbackMode();
      }
      
      // Create cache-busting URL for iOS devices
      const url = isIOS18_3_1 || skipCache 
        ? `/api/auth/me?_t=${Date.now()}` 
        : "/api/auth/me";
      
      // Custom headers for iOS 18.3.1
      const options = isIOS18_3_1 ? {
        headers: {
          'Cache-Control': 'no-cache, no-store, must-revalidate',
          'Pragma': 'no-cache',
          'X-iOS-Version': '18.3.1',
          'X-Device-Special-Handling': 'true',
          'X-Cache-Buster': Date.now().toString()
        }
      } : undefined;
      
      // Attempt normal auth endpoint first
      const response = await apiRequest("GET", url, null, options);
      
      if (response.ok) {
        const userData = await response.json();
        setUser(userData);
        setRetryCount(0); // Reset retry counter on success
        console.log("Auth endpoint succeeded:", userData);
      } else {
        // Try fallback endpoint for iOS 18.3.1 devices
        if (isIOS18_3_1 && retryCount < 3) {
          console.log("Auth endpoint failed, trying fallback");
          try {
            // Try alternative endpoint
            const fallbackResponse = await apiRequest(
              "GET", 
              `/api/fallback-poll?auth=true&_t=${Date.now()}`,
              null,
              options
            );
            
            if (fallbackResponse.ok) {
              const fallbackData = await fallbackResponse.json();
              if (fallbackData.user) {
                setUser(fallbackData.user);
                setRetryCount(0);
                console.log("Fallback auth succeeded:", fallbackData.user);
                return;
              }
            }
            
            console.log("Fallback endpoint also failed");
          } catch (fallbackErr) {
            console.error("Fallback auth error:", fallbackErr);
          }
        }
        
        // Not authenticated, but not an error
        setUser(null);
      }
    } catch (err) {
      console.error("All attempts to get current user failed:", err);
      
      // Only count as error if we've tried multiple times
      if (retryCount >= 2) {
        setError(err instanceof Error ? err : new Error("Failed to check authentication"));
      }
      
      // Automatically retry for iOS devices
      if (isIOS18_3_1 && retryCount < 3) {
        const retryDelay = Math.min(1000 * Math.pow(2, retryCount), 5000);
        console.log(`Will retry auth check in ${retryDelay}ms (attempt ${retryCount + 1}/3)`);
        
        setTimeout(() => {
          setRetryCount(prev => prev + 1);
          checkAuth(true);
        }, retryDelay);
      } else {
        // Not authenticated after retries
        setUser(null);
      }
    } finally {
      setIsLoading(false);
    }
  }, [isLoading, retryCount, isIOS18_3_1, error]);

  // Check auth on mount
  useEffect(() => {
    checkAuth();
  }, [checkAuth]);

  // Enhanced login with iOS 18.3.1 support
  const login = async (username: string, password: string) => {
    try {
      setIsLoading(true);
      setError(null);
      
      // Create options for iOS 18.3.1
      const options = isIOS18_3_1 ? {
        headers: {
          'Cache-Control': 'no-cache, no-store, must-revalidate',
          'Pragma': 'no-cache',
          'X-iOS-Version': '18.3.1',
          'X-Device-Special-Handling': 'true'
        }
      } : undefined;
      
      // If on iOS 18.3.1, try direct endpoint first
      if (isIOS18_3_1) {
        await activateFallbackMode();
        
        try {
          const iosResponse = await apiRequest(
            "POST", 
            `/api/fallback-login?_t=${Date.now()}`, 
            { username, password },
            options
          );
          
          if (iosResponse.ok) {
            const userData = await iosResponse.json();
            setUser(userData);
            
            toast({
              title: "Login Successful",
              description: `Welcome back, ${userData.username}!`,
            });
            
            return;
          }
        } catch (iosErr) {
          console.warn("iOS 18.3.1 specific login failed, trying standard endpoint:", iosErr);
        }
      }
      
      // Standard login endpoint as fallback
      const response = await apiRequest("POST", "/api/auth/login", { username, password }, options);
      
      if (!response.ok) {
        const errorData = await response.json();
        throw new Error(errorData.message || "Login failed");
      }
      
      const userData = await response.json();
      setUser(userData);
      
      toast({
        title: "Login Successful",
        description: `Welcome back, ${userData.username}!`,
      });
    } catch (err) {
      console.error("Login failed:", err);
      setError(err instanceof Error ? err : new Error("Login failed"));
      toast({
        title: "Login Failed",
        description: err instanceof Error ? err.message : "Failed to login",
        variant: "destructive",
      });
      throw err;
    } finally {
      setIsLoading(false);
    }
  };

  // Enhanced logout with iOS 18.3.1 support
  const logout = async () => {
    try {
      setIsLoading(true);
      
      // Create options for iOS 18.3.1
      const options = isIOS18_3_1 ? {
        headers: {
          'Cache-Control': 'no-cache, no-store, must-revalidate',
          'Pragma': 'no-cache',
          'X-iOS-Version': '18.3.1',
          'X-Device-Special-Handling': 'true'
        }
      } : undefined;
      
      // Try iOS 18.3.1 specific endpoint first
      if (isIOS18_3_1) {
        try {
          const iosResponse = await apiRequest(
            "POST", 
            `/api/fallback-logout?_t=${Date.now()}`,
            null,
            options
          );
          
          if (iosResponse.ok) {
            setUser(null);
            
            toast({
              title: "Logged Out",
              description: "You have been successfully logged out",
            });
            
            return;
          }
        } catch (iosErr) {
          console.warn("iOS 18.3.1 specific logout failed, trying standard endpoint:", iosErr);
        }
      }
      
      // Standard logout endpoint as fallback
      const response = await apiRequest("POST", "/api/auth/logout", null, options);
      
      if (!response.ok) {
        const errorData = await response.json();
        throw new Error(errorData.message || "Logout failed");
      }
      
      setUser(null);
      
      toast({
        title: "Logged Out",
        description: "You have been successfully logged out",
      });
    } catch (err) {
      console.error("Logout failed:", err);
      setError(err instanceof Error ? err : new Error("Logout failed"));
      toast({
        title: "Logout Failed",
        description: err instanceof Error ? err.message : "Failed to logout",
        variant: "destructive",
      });
    } finally {
      setIsLoading(false);
    }
  };

  // Enhanced register with iOS 18.3.1 support
  const register = async (userData: RegisterData) => {
    try {
      setIsLoading(true);
      setError(null);
      
      // Create options for iOS 18.3.1
      const options = isIOS18_3_1 ? {
        headers: {
          'Cache-Control': 'no-cache, no-store, must-revalidate',
          'Pragma': 'no-cache',
          'X-iOS-Version': '18.3.1',
          'X-Device-Special-Handling': 'true'
        }
      } : undefined;
      
      // If on iOS 18.3.1, try direct endpoint first
      if (isIOS18_3_1) {
        await activateFallbackMode();
        
        try {
          const iosResponse = await apiRequest(
            "POST", 
            `/api/fallback-register?_t=${Date.now()}`, 
            userData,
            options
          );
          
          if (iosResponse.ok) {
            const newUser = await iosResponse.json();
            setUser(newUser);
            
            toast({
              title: "Registration Successful",
              description: `Welcome, ${newUser.username}!`,
            });
            
            return;
          }
        } catch (iosErr) {
          console.warn("iOS 18.3.1 specific registration failed, trying standard endpoint:", iosErr);
        }
      }
      
      // Standard registration endpoint as fallback
      const response = await apiRequest("POST", "/api/auth/register", userData, options);
      
      if (!response.ok) {
        const errorData = await response.json();
        throw new Error(errorData.message || "Registration failed");
      }
      
      const newUser = await response.json();
      setUser(newUser);
      
      toast({
        title: "Registration Successful",
        description: `Welcome, ${newUser.username}!`,
      });
    } catch (err) {
      console.error("Registration failed:", err);
      setError(err instanceof Error ? err : new Error("Registration failed"));
      toast({
        title: "Registration Failed",
        description: err instanceof Error ? err.message : "Failed to register",
        variant: "destructive",
      });
      throw err;
    } finally {
      setIsLoading(false);
    }
  };

  // Retry auth function that can be called from components
  const retryAuth = async () => {
    setRetryCount(0);
    await checkAuth(true);
  };

  return (
    <AuthContext.Provider value={{ 
      user, 
      isLoading, 
      error, 
      login, 
      logout, 
      register,
      retryAuth,
      lastAttemptTime
    }}>
      {children}
    </AuthContext.Provider>
  );
}

export function useAuth() {
  const context = useContext(AuthContext);
  if (context === null) {
    throw new Error("useAuth must be used within an AuthProvider");
  }
  return context;
}

export default useAuth;